@@ -34,8 +34,13 @@ def parse_config(data):
 include_preflight = 0
 
 class SyncState(object):
-  def __init__(self, proc, quiet):
+  def __init__(self, proc, server, quiet):
+    st = server.split("@", 1)
+    if len(st) < 2:
+      st.append("default")
     self.proc = proc
+    self.server = st[0]
+    self.context = st[1]
     self.quiet = quiet
     self.sources = []
     self.data = ""
@@ -60,24 +65,31 @@ class SyncState(object):
         return
       source = data[:pos]
       state = data[pos+2:]
-      if not source in self.sources:
-        if state.startswith("starting"):
-          self.sources.append(source)
-          if include_preflight:
-            self.stages = len(self.sources) * 2
+
+      if source[:1] == "@":
+        context, source = source[1:].split("/", 1)
+      else:
+        context = "default"
+
+      if context == self.context:
+        if not source in self.sources:
+          if state.startswith("starting"):
+            self.sources.append(source)
+            if include_preflight:
+              self.stages = len(self.sources) * 2
+            else:
+              self.stages = len(self.sources)
           else:
-            self.stages = len(self.sources)
-        else:
-          return
-      source_idx = self.sources.index(source)
-      if state.startswith("preparing"):
-        if include_preflight:
-          self.stage = source_idx
-      elif state.startswith("started"):
-        if include_preflight:
-          self.stage = len(self.sources) + source_idx
-        else:
-          self.stage = source_idx
+            return
+        source_idx = self.sources.index(source)
+        if state.startswith("preparing"):
+          if include_preflight:
+            self.stage = source_idx
+        elif state.startswith("started"):
+          if include_preflight:
+            self.stage = len(self.sources) + source_idx
+          else:
+            self.stage = source_idx
   def watches(self):
     if self.proc is None:
       return []
@@ -124,7 +136,19 @@ class SyncEvolution(object):
     self.path = "syncevolution"
     self.quiet = quiet
   def launch(self, args):
+#    print "syncevolution", " ".join(args)
     return subprocess.Popen([self.path] + args, stdout=subprocess.PIPE)
+  def get_version(self):
+    p = self.launch(["--version"])
+    data = p.communicate()[0]
+    lines = data.splitlines()
+    ltoks = lines[0].split(" ")
+    return ltoks[1]
+  def has_version(self, ver):
+    v = self.get_version()
+    stoks = v.split(".")
+    dtoks = ver.split(".")
+    return stoks >= dtoks
   def get_servers(self):
     p = self.launch(["--print-servers"])
     data = p.communicate()[0]
@@ -166,7 +190,8 @@ class SyncEvolution(object):
     p = self.launch(["--print-config", "--quiet", "--template", template])
     data = p.communicate()[0]
     return parse_config(data)
-
+  def has_contexts(self):
+    return self.has_version("1.0")
   def get_session_dirs(self, server):
     p = self.launch(["--print-sessions", "--quiet", server])
     data = p.communicate()[0]
@@ -177,11 +202,15 @@ class SyncEvolution(object):
     sessions = []
     for dir in self.get_session_dirs(server):
       status_fn = os.path.join(dir, "status.ini")
-      log_fn = os.path.join(dir, "sysynclib_linux.html")
       try:
         info_data = open(status_fn, "r").read()
       except:
         continue
+      log_fn = os.path.join(dir, "syncevolution-log.html")
+      if not os.access(log_fn, os.F_OK):
+        log_fn2 = os.path.join(dir, "sysynclib_linux.html")
+        if os.access(log_fn2, os.F_OK):
+          log_fn = log_fn2
       info = parse_config(info_data)
       if info.has_key("start"):
         start_time = int(info["start"].split(",")[0])
@@ -204,6 +233,10 @@ class SyncEvolution(object):
       if line == "":
         pass
       elif line.startswith(" "):
+        if not line.startswith("   "):
+          # source listing apparently complete
+          del ret[key]
+          break
         epos = line.rfind(")")
         spos = line.rfind(" (")
         if spos < 0 or epos < 0:
@@ -215,14 +248,15 @@ class SyncEvolution(object):
       else:
         spos = line.rfind(" = ")
         if spos < 0:
-          # source listing apparently complete
-          break
-        epos = line.find(":", spos)
-        key = line[spos+3:epos]
+          epos = line.find(":")
+          key = line[:epos]
+        else:
+          epos = line.find(":", spos)
+          key = line[spos+3:epos]
         current = ret.setdefault(key, [])
     return ret
 
-  def create_server(self, server, template, username=None, password=None, config=None):
+  def create_server(self, server, template, username=None, password=None, config=None, sources=None):
     args = ["--configure"]
     if not template is None:
       args.extend(["--template", template])
@@ -238,13 +272,23 @@ class SyncEvolution(object):
           continue
         args.extend(["--sync-property", key + "=" + value])
     args.append(server)
+    # note that without a template, we need another step.
+    if not sources is None and not template is None:
+      args.extend(sources)
     p = self.launch(args)
     data = p.communicate()[0]
     if not source_config is None:
       self.configure_server(server, {"sources": source_config})
+    if not sources is None and template is None:
+      # second step of templateless configuration
+      args = ["--configure"]
+      args.append(server)
+      args.extend(sources)
+      p = self.launch(args)
+      data = p.communicate()[0]
 
   def delete_server(self, server):
-    # syncevolution can't do this on its own, delete the directory ourselves
+    # syncevolution 0.9 can't do this on its own, delete the directory ourselves
     # delete session caches
     for dir in self.get_session_dirs(server):
       try:
@@ -260,7 +304,12 @@ class SyncEvolution(object):
       return True
     except:
       return False
-
+  def delete_context(self, context):
+    # For contexts, there's no direct way to determine which path
+    # to rmtree, so leavee the destruction to syncevolution
+    # (contexts are a syncevolution 1.0 feature anyway)
+    p = self.launch(["--remove", "@" + context])
+    data = p.communicate()[0]
   def configure_server(self, server, server_config):
     source_config = None
     args = ["--configure"]
@@ -305,7 +354,7 @@ class SyncEvolution(object):
       print self.path, " ".join(args)
     p = self.launch(args)
     set_nonblock(p.stdout)
-    return SyncState(p, self.quiet)
+    return SyncState(p, server, self.quiet)
   def synchronize_status(self, state):
     ret = {"result": state.result()}
     if state.errlog is not None:
@@ -313,7 +362,10 @@ class SyncEvolution(object):
       statusfile = os.path.join(logdir, "status.ini")
       status = parse_config(open(statusfile, "r").read())
       code = int(status["status"])
-      msg = status["error"]
+      if status.has_key("error"):
+        msg = status["error"]
+      else:
+        msg = None
       pos = msg.find(" (")
       if pos >= 0:
         # to keep the message short enough to fit on display (and not confuse user),
