@@ -440,6 +440,7 @@ class SyncGUI(object):
       alarm.delete_event(server_cookie)
     self.sync.delete_server(server)
     if self.sync_is_local(server_config) != 0:
+      url = server_config.get("syncURL")
       self.sync.delete_context(url[9:])
   def do_delete_context(self, context):
     if not self.sync.has_contexts():
@@ -659,7 +660,7 @@ class SyncGUI(object):
         config = std_config.copy()
         config["syncURL"] = "local://@" + src_context
         for key in ["WebURL", "PeerName", "consumerReady"]:
-          if temp_config.has_key[key]:
+          if temp_config.has_key(key):
             config[key] = temp_config[key]
         for name in disabled_sources:
           config1 = {"sync": "disabled"}
